/*
 * Copyright (c) 2021, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <msp430.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include "tps272c45evm.h"

/**
 * tps272c45_adc_polling.c
 */
volatile bool en1On = false;
volatile bool en2On = false;
volatile uint16_t adcConvRes = 0;

int main(void)
{
    /* Stop/Hold the Watchdog */
    WDTCTL = WDTPW | WDTHOLD;

    /* Configuring all digital signals */
    DIA_EN_OUT &= ~DIA_EN_PIN;
    DIA_EN_DIR |= DIA_EN_PIN;
    SEL_OUT &= ~SEL_PIN;
    SEL_DIR |= SEL_PIN;
    EN1_OUT &= ~EN1_PIN;
    EN1_DIR |= EN1_PIN;
    EN2_OUT &= ~EN2_PIN;
    EN2_DIR |= EN2_PIN;

    /* Configuring the fault interrupt */
    FAULT_IES &= ~FAULT_PIN;
    FAULT_IFG &= ~FAULT_PIN;
    FAULT_IE |= FAULT_PIN;

    /* Configuring Push Buttons */
    /* S2 */
    P2REN |= BIT1;
    P2OUT |= BIT1;
    P2IES |= BIT1;
    P2IFG &= ~BIT1;
    P2IE |= BIT1;

    /* S1 */
    P1REN |= BIT1;
    P1OUT |= BIT1;
    P1IES |= BIT1;
    P1IFG &= ~BIT1;
    P1IE |= BIT1;

    /* Configuring LEDs */
    P1OUT &= ~BIT0;
    P1DIR |= BIT0;
    P4OUT &= ~BIT7;
    P4DIR |= BIT7;

    /* Configuring the fault timer */
    TA0CCTL0 = CCIE;
    TA0CCR0 = 50000;

    /* Configuring the ADC for conversion */
    SNS_SEL |= SNS_PIN;
    ADC12CTL0 = ADC12ON + ADC12SHT0_2;       // Turn on ADC12, set sampling time
    ADC12CTL1 = ADC12SHP;  // Use sampling timer
    ADC12MCTL0 |= (ADC12INCH_5 | ADC12EOS);  // Use channel A5
    ADC12CTL0 |= ADC12ENC;                   // Enable conversions
    ADC12IE = BIT0;                          // Enable interrupts

    /* Enabling interrupts and sleeping */
    while(1)
    {
        /* Start the ADC Conversion and wait for completion interrupt */

        /* Enabling the diagnostic block and delaying */
        DIA_EN_OUT |= DIA_EN_PIN;
        __delay_cycles(1000);

        /* Starting the conversion */
        ADC12CTL0 |= ADC12SC;

        /* Waiting for the interrupt */
        __bis_SR_register(LPM0_bits + GIE);

        /* Disabling diagnostic block */
        DIA_EN_OUT &= ~DIA_EN_PIN;

        /* Set breakpoint here to record result */
        __no_operation();
        printf("\nADC raw result 0x%x", adcConvRes);

        /* Delaying between measurements */
        __delay_cycles(1000000);
    }
}

#pragma vector=PORT1_VECTOR
__interrupt void Port_1(void)
{
    switch (__even_in_range(P1IV, 16))
    {
        case P1IV_NONE:
            break;   // Vector  0: no interrupt
        case P1IV_P1IFG0:
            break;   // Vector  2: P1.0
        case P1IV_P1IFG1:
            P4OUT ^= BIT7;    // Vector  4: P1.1 (EN2)
            en2On = !en2On;
            EN2_OUT ^= EN2_PIN;
            break;
        case P1IV_P1IFG2:
            break;   // Vector  6: P1.2
        case P1IV_P1IFG3:
            break;   // Vector  8: P1.3
        case P1IV_P1IFG4:
            FAULT_IE &= ~FAULT_PIN;   // Vector 10: P1.4 (Fault)
            TA0CTL = TASSEL_2 + MC_1 + TACLR;
            break;
        case P1IV_P1IFG5:
            break;   // Vector 12: P1.5
        case P1IV_P1IFG6:
            break;   // Vector 14: P1.6
        case P1IV_P1IFG7:
            break;   // Vector 16: P1.7
        default:
            break;
    }
}

#pragma vector=PORT2_VECTOR
__interrupt void Port_2(void)
{
    switch (__even_in_range(P2IV, 16))
    {
        case P2IV_NONE:
            break;   // Vector  0: no interrupt
        case P2IV_P2IFG0:
            break;   // Vector  2: P2.0
        case P2IV_P2IFG1:
            P1OUT ^= BIT0;    // Vector  4: P2.1 (EN1)
            en1On = !en1On;
            EN1_OUT ^= EN1_PIN;
            break;
        case P2IV_P2IFG2:
            break;   // Vector  6: P2.2
        case P2IV_P2IFG3:
            break;   // Vector  8: P2.3
        case P2IV_P2IFG4:
            break;   // Vector 10: P2.4
        case P2IV_P2IFG5:
            break;   // Vector 12: P2.5
        case P2IV_P2IFG6:
            break;   // Vector 14: P2.6
        case P2IV_P2IFG7:
            break;   // Vector 16: P2.7
        default:
            break;
    }
}

// Timer0 A0 interrupt service routine
#pragma vector=TIMER0_A0_VECTOR
__interrupt void TIMER0_A0_ISR(void)
{
    /* If the fault is gone, return to normal operation */
    if(FAULT_IN | FAULT_PIN)
    {
        TA0CTL = 0;

        if(en1On == true)
        {
            P1OUT |= BIT0;
        }
        else
        {
            P1OUT &= ~BIT0;
        }

        if (en2On == true)
        {
            P4OUT |= BIT7;
        }
        else
        {
            P4OUT &= ~BIT7;
        }
    }
}

#pragma vector = ADC12_VECTOR
__interrupt void ADC12_ISR(void)
{
    switch (__even_in_range(ADC12IV, 34))
    {
        case 0:
            break;                           // Vector  0:  No interrupt
        case 2:
            break;                           // Vector  2:  ADC overflow
        case 4:
            break;                           // Vector  4:  ADC timing overflow
        case 6:
            /* Enable main loop */
            adcConvRes = ADC12MEM0;
            __bic_SR_register_on_exit(LPM0_bits);
            break;                           // Vector  6:  ADC12IFG0 (SNS)
        case 8:
            break;                           // Vector  8:  ADC12IFG1
        case 10:
            break;                           // Vector 10:  ADC12IFG2
        case 12:
            break;                           // Vector 12:  ADC12IFG3
        case 14:
            break;                           // Vector 14:  ADC12IFG4
        case 16:
            break;                          // Vector 16:  ADC12IFG5
        case 18:
            break;                           // Vector 18:  ADC12IFG6
        case 20:
            break;                           // Vector 20:  ADC12IFG7
        case 22:
            break;                           // Vector 22:  ADC12IFG8
        case 24:
            break;                           // Vector 24:  ADC12IFG9
        case 26:
            break;                           // Vector 26:  ADC12IFG10
        case 28:
            break;                           // Vector 28:  ADC12IFG11
        case 30:
            break;                           // Vector 30:  ADC12IFG12
        case 32:
            break;                           // Vector 32:  ADC12IFG13
        case 34:
            break;                           // Vector 34:  ADC12IFG14
        default:
            break;
    }
}

