/*
 * Copyright (c) 2021, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <msp430.h>
#include <stdbool.h>
#include <stdint.h>
#include "tps272c45evm.h"

/**
 * tps272c45_comparator.c
 */
int main(void)
{
    /* Stop/Hold the Watchdog */
    WDTCTL = WDTPW | WDTHOLD;

    /* Configuring all digital signals */
    DIA_EN_OUT &= ~DIA_EN_PIN;
    DIA_EN_DIR |= DIA_EN_PIN;
    SEL_OUT &= ~SEL_PIN;
    SEL_DIR |= SEL_PIN;
    EN1_OUT &= ~EN1_PIN;
    EN1_DIR |= EN1_PIN;

    /* Configuring Push Button */
    /* S1 */
    P2REN |= BIT1;
    P2OUT |= BIT1;
    P2IES |= BIT1;
    P2IFG &= ~BIT1;
    P2IE |= BIT1;

    /* Configuring LED */
    P1OUT &= ~BIT0;
    P1DIR |= BIT0;

    /* Enabling diagnostics and waiting for a delay */
    DIA_EN_OUT |= DIA_EN_PIN;
    __delay_cycles(1000);

    /* Setup Comparator B */
    CBCTL0 |= CBIPEN + CBIPSEL_5; // Enable V+, input channel CB5
    CBCTL1 |= CBPWRMD_1;          // normal power mode
    CBCTL2 |= CBRSEL;             // VREF is applied to -terminal

    /* VCC (3.3V) * (8/32)  reference select. This equates to approximately
     * 1A with the 1k resistor on the TPS272C45 SNS pin *.
     */
    CBCTL2 |= CBRS_1 | CBREF0_7 | CBREF1_6;
    CBCTL3 |= BIT0;               // Input Buffer Disable

    __delay_cycles(75);           // delay for the reference to settle

    CBINT &= ~(CBIFG + CBIIFG);   // Clear any errant interrupts
    CBINT |= CBIE;   // Enable CompB Interrupt on rising edge of CBIFG (CBIES=0)
    CBCTL1 |= CBON;               // Turn On ComparatorB

    __bis_SR_register(LPM4_bits + GIE); // Enter LPM4
}

#pragma vector=PORT2_VECTOR
__interrupt void Port_2(void)
{
    switch (__even_in_range(P2IV, 16))
    {
        case P2IV_NONE:
            break;   // Vector  0: no interrupt
        case P2IV_P2IFG0:
            break;   // Vector  2: P2.0
        case P2IV_P2IFG1:
            EN1_OUT ^= EN1_PIN;
            break;
        case P2IV_P2IFG2:
            break;   // Vector  6: P2.2
        case P2IV_P2IFG3:
            break;   // Vector  8: P2.3
        case P2IV_P2IFG4:
            break;   // Vector 10: P2.4
        case P2IV_P2IFG5:
            break;   // Vector 12: P2.5
        case P2IV_P2IFG6:
            break;   // Vector 14: P2.6
        case P2IV_P2IFG7:
            break;   // Vector 16: P2.7
        default:
            break;
    }
}

/* Comparator ISR */
#pragma vector=COMP_B_VECTOR
__interrupt void Comp_B_ISR (void)
{
    CBCTL1 ^= CBIES;              // Toggles interrupt edge
    CBINT &= ~CBIFG;              // Clear Interrupt flag
    P1OUT ^= 0x01;                // Toggle P1.0
}
